

--Libs
Engine = {};
LevelMatrix = {};
Tileset = {};

NoGameScene = {};
--Constants
Engine.ER_LOAD_NF = 'Inital function in CurrentScene is not defined.';
Engine.ER_UPDATE_NF = 'Update function in CurrentScene is not defined.';
Engine.ER_DRAW_NF = 'Draw function in CurrentScene is not defined.';
Engine.ER_SCN_NF = 'The scene is not defined!';
Engine.ER_SCN_FREE = "Function 'free()' of the scene is not defined."

--State
Engine.ErrorActive = false;
Engine.ErrorString = '';

Engine.CurrentLevel = {};
Engine.CameraX = 0;
Engine.CameraY = 0;
Engine.MouseX = 0;
Engine.MouseY = 0;

LevelMatrix.MapLoaded = ''
AlphaTransistion = 0;
---- (!) Global Game functions
function Engine.Init()
	iTiles = love.graphics.newImage('BaseTileset.png');
	sTiles = {1, 2, 3, 4, 5, 6, 9, 10, 11, 12, 13, 14, 15, 17, 18, 19, 20, 28, 34, 37};
	tTiles = Tileset.Create(iTiles, 32, 32, sTiles);
	iVingette = love.graphics.newImage('Graphics/vingette.png')
	gameFont = love.graphics.newFont('Russo_One.ttf', 24)

	Inv = InventoryInit();
end

function Engine.ViewPosition(x, y)
	Engine.CameraX = math.floor(-x);
	Engine.CameraY = math.floor(-y);
end

function Engine.Update(dt)
	Engine.MouseX = love.mouse.getX() - Engine.CameraX;
	Engine.MouseY = love.mouse.getY() - Engine.CameraY;
	if(CurrentTransform ~= CurrentScene) then
		TimeTransistionBetweenScenes = TimeTransistionBetweenScenes + dt
		if(TimeTransistionBetweenScenes > TimerTransistionBetweenScenes) then
			Engine.ChangeScene(CurrentTransform)
			TimeTransistionBetweenScenes = 0
		end
	end
end

function Engine.Error(Str)
	Engine.ErrorString = Str;
	Engine.ErrorActive = true;
end

function Engine.ErrorDisplay()
	if(Engine.ErrorActive) then
		love.graphics.setColor(33, 33, 33, 255);
		love.graphics.rectangle('fill', 0, 0, love.window.getWidth(), love.window.getHeight());
		love.graphics.setColor(255, 255, 255, 255);
		love.graphics.print("ERROR: \n" + Engine.ErrorString, 16, 16);
	end
end

function Engine.GetDirection(x1, y1, x2, y2) 
	local yy = y1-y2;
	local xx = x1-x2;
	return math.atan2(yy, xx) + math.pi;
end

function Engine.GetDistantion(x1, y1, x2, y2)
	return math.sqrt((x1 - x2)^2 + (y1 - y2)^2)
end

function Engine.Gui()
	love.graphics.translate(-Engine.CameraX, -Engine.CameraY)
	love.graphics.setFont(gameFont);

	love.graphics.setColor(0, 0, 0, 255 * AlphaTransistion)
	love.graphics.rectangle('fill', 0, 0, love.window.getWidth(), love.window.getHeight())
	if(Loading == false) then
		if(CurrentScene == Game) then
			if(Pause == true) then
				love.graphics.setColor(0, 0, 0, 128)
				love.graphics.rectangle('fill', 0, 0, love.window.getWidth(), love.window.getHeight())
				love.graphics.setColor(255, 255, 255, 255)
				if(Inv.Opened == false) then 
					love.graphics.print('Pause\n\n Y - to main menu\n Esc - Return', 32, 32)
				end
			else
			
				love.graphics.setColor(0, 0, 0, 128)
				love.graphics.rectangle('fill', 0, 0, love.window.getWidth(), 64);
				love.graphics.rectangle('fill', 0, love.window.getHeight() - 64, love.window.getWidth(), 64);
				love.graphics.draw(iVingette, love.graphics.newQuad(0, 0, 800, 480, 800, 480), 0, 0, 0, love.window.getWidth()/800, love.window.getHeight()/480)
				if(player)then
					if(player.HP > 1) then
						love.graphics.setColor(255, 255, 255, 255)
						love.graphics.rectangle('line', 16, 16, 256, 32)
						love.graphics.print(player.WeaponStats[player.Weapon][5], love.window.getWidth(), love.window.getHeight() - 64, 0, 1, 1, gameFont:getWidth(player.WeaponStats[player.Weapon][5]), 0)
						love.graphics.print("Scores: " + Scores, 0, love.window.getHeight() - 64)
						love.graphics.print("Sector: " + Sector, 0, love.window.getHeight() - 44)
						love.graphics.print(Coins, 400 - gameFont:getWidth(tostring(Coins))/2, love.window.getHeight() - 44, 0, 1.5, 1.5, gameFont:getWidth(tostring(Coins))/2, gameFont:getHeight()/2)
						love.graphics.rectangle('fill', 20, 20, (256-8) * (player.HP/100), 32-8)
					else
						love.graphics.setColor(255, 0, 0, 128)
						love.graphics.rectangle('fill', 0, 0, love.window.getWidth(), love.window.getHeight())
						love.graphics.setColor(255, 255, 255, 255)
						local StringLose = 'You died\nYour Scores: '+Scores+' x '+(Sector + 1)+' = '+Scores*(Sector + 1) + '\nPress ESC to return to MainMenu'
						love.graphics.print(StringLose, love.window.getWidth()/2, love.window.getHeight()/2, 0, 1, 1, gameFont:getWidth(StringLose)/2, gameFont:getHeight()/2)
					end
				end
			end
			if(player and player.HP > 1) then Inv.Draw(); else Inv.Opened = false; end
		end
	else
		love.graphics.setColor(0, 0, 0, 255)
		love.graphics.rectangle('fill', 0, 0, love.window.getWidth(), love.window.getHeight())
		love.graphics.setColor(255, 255, 255, 255)
		love.graphics.draw(Sprite[8][1], LoadingQuad, 16 + gameFont:getWidth('Loading...') + 24, love.window.getHeight() - 16, 0, 1, 1, 24, 24)
		love.graphics.print('Loading...', 16, love.window.getHeight() - 32)
	end
end

function Engine.ChangeScene(scene)
	if(scene) then
		if(CurrentScene.free ~= nil) then
			CurrentScene.free();
			CurrentScene = scene;
			CurrentTransform = CurrentScene;
			CurrentScene.load();
		else
			Engine.Error(Engine.ER_SCN_FREE)
		end	
	else
		Engine.Error(Engine.ER_SCN_NF)	
	end
end

function Engine.CheckMouseBox(x, y, w, h)
	local mx = love.mouse.getX();
	local my = love.mouse.getY();
	if(mx > x and mx < x + w and my > y and my < y + h) then
		return true;
	end
	return false;
end

----Matrix of level
function LevelMatrix.Load(filename)
	unrequire(LevelMatrix.MapLoaded)
	Level = nil
	require(filename);
	LevelMatrix.MapLoaded = filename;
end

function LevelMatrix.Display(tileid)
	if(Level ~= nil) then
		for j, v in ipairs(Level) do
			if(j > math.floor((-32-Engine.CameraY)/32) and j < math.floor((-Engine.CameraY + love.window.getHeight())/32) + 1) then
				for i = math.floor(-Engine.CameraX/32), math.floor((-Engine.CameraX + love.window.getWidth())/32) do
					if(Level[j] ~= nil and Level[j][i] ~= nil) then
						local d = Level[j][i]
						if(d ~= 0) then
							love.graphics.setColor(255, 255, 255, 255)
							local quad = love.graphics.newQuad(tileid[d][1], tileid[d][2], 32, 32, 256, 192)
							love.graphics.draw(tileid.__imageid, quad, 32 * i, 32 * j)
						end
					end
				end
			end
		end
	end
end

function LevelMatrix.HitTest(tileid, x, y)
	local xx = math.floor(x/tileid.__tilewidth);
	local yy = math.floor(y/tileid.__tileheight);

	if(xx > 0 and yy > 0) then
		if(Level[yy] ~= nil and Level[yy][xx] ~= nil and tileid[Level[yy][xx]] ~= nil and tileid[Level[yy][xx]][3] ~= nil)then
			return tileid[Level[yy][xx]][3]
		end
	end
	return false;
end

function LevelMatrix.GetCell(tileid, x, y)
	local xx = math.floor(x/tileid.__tilewidth);
	local yy = math.floor(y/tileid.__tileheight);

	if(xx > 0 and yy > 0) then
		if(Level[yy] ~= nil and Level[yy][xx] ~= nil)then
			return Level[yy][xx]
		end
	end
	return 0;
end

--Tileset functions
function Tileset.Create(imageid, tilewidth, tileheight, tabSolid)
	local indexes = {};
	indexes.__imageid = imageid;
	indexes.__tilewidth = tilewidth;
	indexes.__tileheight = tileheight;

	local numinline = math.floor(imageid: getWidth() / tilewidth);
	local lines = math.floor(imageid: getHeight() / tileheight);
	for j = 0, lines - 1 do
		for i = 0, numinline - 1 do
			local tab = {i * tilewidth, j * tileheight};

			for m, t in ipairs(tabSolid) do
				if(t == numinline * j + 1 + i) then
					tab[3] = true;
					break;
				end
				tab[3] = false;
			end
			table.insert(indexes, tab);
		end
	end
	return indexes;
end

function Tileset.DrawTile(tileid, index, x, y)
	local tile = tileid[index];
	local quad = love.graphics.newQuad(tile[1], tile[2], tileid.__tilewidth, tileid.__tileheight, tileid.__imageid:getWidth(), tileid.__imageid:getHeight())
	love.graphics.draw(tileid.__imageid, quad, x, y);
end








----No Game Scene
function NoGameScene.load()
	
end

function NoGameScene.update(dt)

end

function NoGameScene.draw()
	love.graphics.setColor(33, 33, 33, 255);
	love.graphics.rectangle('fill', love.window.getWidth()/2 - 320, love.window.getHeight()/2 - 240, 640, 480);

	love.graphics.setColor(255, 255, 255, 255);
	love.graphics.print('This is empty scene. Change variable name "CurrentScene".', love.window.getWidth()/2 - 320, love.window.getHeight()/2 - 240);
end

function unrequire(m)
	package.loaded[m] = nil
	_G[m] = nil
end

--Inventory
function InventoryInit()
	local inv = {}
	inv.Items = {0, 0, 0, 0, 0, 0}
	inv.ItemsCount = {0, 0, 0, 0, 0, 0}
	inv.ItemsName = {"", "", "", "", "", ""}
	inv.ItemsFunc = {0, 0, 0, 0, 0, 0}
	inv.Selected = {false, false, false, false, false, false}
	
	inv.Opened = false;
	inv.AnimOpened = 0;

	local Drag = false;

	inv.Draw = function()
		local Mx, My = love.mouse.getX(), love.mouse.getY();
		local Dist = 192 * inv.AnimOpened
		local SelSize = 8;
		love.graphics.setColor(255, 255, 255, 128*inv.AnimOpened)
		love.graphics.circle('fill', love.window.getWidth()/2, love.window.getHeight()/2, 128*inv.AnimOpened, 32)
		if(inv.Opened)then
			inv.AnimOpened = inv.AnimOpened - (inv.AnimOpened - 1)/8
			Pause = true;
		else
			inv.AnimOpened = inv.AnimOpened - (inv.AnimOpened - 0)/8
		end
		for i = 0, 5 do
			local Angle = (math.pi*2)/6 ;
			Rotate = Angle*i + (90/-180*math.pi)*inv.AnimOpened
			
			love.graphics.setColor(255, 255, 255, 128*inv.AnimOpened)
			
			if(inv.Selected[i+1] == false) then
				SelSize = 0;
			else
				SelSize = 8;
			end
			love.graphics.circle('fill', love.window.getWidth()/2 + math.cos(Rotate)*Dist, love.window.getHeight()/2 + math.sin(Rotate)*Dist, (32 + SelSize)*inv.AnimOpened, 16)
			
			if(inv.Items[i+1] ~= 0) then
				local w, h = Item[inv.Items[i+1]]:getWidth(), Item[inv.Items[i+1]]:getHeight()
				love.graphics.setColor(255, 255, 255, 255*inv.AnimOpened)
				love.graphics.draw(Item[inv.Items[i+1]], love.graphics.newQuad(0, 0, w, h, w, h), love.window.getWidth()/2 + math.cos(Rotate)*Dist, love.window.getHeight()/2 + math.sin(Rotate)*Dist, 5*inv.AnimOpened, inv.AnimOpened, inv.AnimOpened, w/2, h/2)
				love.graphics.print(inv.ItemsCount[i+1], love.window.getWidth()/2 + math.cos(Rotate)*Dist + 4, love.window.getHeight()/2 + math.sin(Rotate)*Dist + 4)
			end

			if(inv.Opened) then
				if(Engine.GetDistantion(love.window.getWidth()/2 + math.cos(Rotate)*Dist, love.window.getHeight()/2 + math.sin(Rotate)*Dist, Mx, My) < 32) then
					inv.Selected[i+1] = true;
					if(Drag == false) then CurrentItem = i + 1 end
				else
					inv.Selected[i+1] = false;
				end

				if(love.mouse.isDown('l')) then
					if(inv.Selected[i+1]) then Drag = true; end
					if(inv.Items[CurrentItem] ~= 0 and Drag) then
						love.graphics.setColor(30, 30, 30, 255)
						love.graphics.print(inv.ItemsName[CurrentItem] + "\nDrag here to use!", love.window.getWidth()/2 - gameFont:getWidth(inv.ItemsName[CurrentItem] + "\nDrag here to use!")/2, love.window.getHeight()/2 - gameFont:getHeight())
						love.graphics.circle('fill', Mx, My, 32, 16)
						love.graphics.setColor(255, 255, 255, 255)
						local w, h = Item[inv.Items[CurrentItem]]:getWidth(), Item[inv.Items[CurrentItem]]:getHeight()
						love.graphics.draw(Item[inv.Items[CurrentItem]], love.graphics.newQuad(0, 0, w, h, w, h), Mx, My, 5*inv.AnimOpened, 1, 1, w/2, h/2)
						love.graphics.print(inv.ItemsCount[CurrentItem], Mx, My)
					end
				else
					if(Drag) then
						if(inv.Selected[i + 1]) then
							local NewItem, NewCounts, NewName, NewFunc = inv.Items[i + 1], inv.ItemsCount[i + 1], inv.ItemsName[i + 1], inv.ItemsFunc[i + 1]
							inv.Items[i + 1] = inv.Items[CurrentItem];
							inv.ItemsCount[i + 1] = inv.ItemsCount[CurrentItem];
							inv.ItemsName[i + 1] = inv.ItemsName[CurrentItem];
							inv.ItemsFunc[i + 1] = inv.ItemsFunc[CurrentItem];

							inv.Items[CurrentItem] = NewItem;
							inv.ItemsCount[CurrentItem] = NewCounts;
							inv.ItemsName[CurrentItem] = NewName;
							inv.ItemsFunc[CurrentItem] = NewFunc;
							Drag = false;
						end

						if(Engine.GetDistantion(love.window.getWidth()/2, love.window.getHeight()/2, Mx, My) < 128) then
							if(inv.Items[CurrentItem] ~= 0 and inv.ItemsCount[CurrentItem] > 0) then
								inv.ItemsFunc[CurrentItem]();
								inv.ItemsCount[CurrentItem] = inv.ItemsCount[CurrentItem] - 1;
							end
							Drag = false;
						end
					end
				end

				if(inv.ItemsCount[i + 1] < 1) then
					inv.Items[i + 1] = 0;
				end
			end

		end
		love.graphics.setColor(255, 255, 255, 255)
		love.graphics.print("Inventory\nEsc - to return", love.window.getWidth()/2 - 256, love.window.getHeight()/2 - 192-32, (5/-180*math.pi)*inv.AnimOpened, inv.AnimOpened, inv.AnimOpened, gameFont:getWidth("Inventory")/2, gameFont:getHeight()/2)
	end

	return inv;
end